/* From Oracle Scripts, O Reilly and Associates, Inc. */
/* Copyright 1998 by Brian Lomasky, DBA Solutions, Inc., */
/* lomasky@earthlink.net */

select floor(( ? - 113 - (23 * ? )) * (1-( ? /100))) ads from dual
